@@ -9,7 +9,7 @@ pub trait Set {
     fn contains(&self, p: &Array1<f64>) -> bool;
 
     /// Returns a bounding box for the set.
-    /// This function is used for point process simulation by rejection, but can also be used to implement a Monte Carlo estimation of the set"s measure.
+    /// This function is used for point process simulation by rejection, but can also be used to implement a Monte Carlo estimation of the set's measure.
     fn bounding_box(&self) -> Array2<f64>;
 }
 
@@ -30,29 +30,20 @@ impl Set for Rectangle {
     fn contains(&self, p: &Array1<f64>) -> bool {
         assert_eq!(p.len(), self.0.shape()[0]);
         
-        // check if p is further away than the closer point
-        let further = self.0.iter().zip(p.iter())
-            .fold(true, |acc: bool, (v,w)| {
-                acc & (w > v)
-            });
-        
-        // check if p is closer than the far point
-        let closer = self.1.iter().zip(p.iter())
-            .fold(true, |acc: bool, (v,w)| {
-                acc & (w < v)
-            });
-
-        // if both conditions are true then we're in the rectangle
-        further & closer
+        for (x,(l,h)) in p.iter().zip(self.0.iter().zip(self.1.iter())) {
+            if (x < l) | (x > h) {
+                return false
+            }
+        }
+        true
     }
 
     fn bounding_box(&self) -> Array<f64, Ix2> {
-        let mut result = unsafe {
-            Array::uninitialized((self.0.shape()[0], 2))
-        };
-        
-        result.slice_mut(s![0,..]).assign(&self.0);
-        result.slice_mut(s![1,..]).assign(&self.1);
+        let d = self.0.shape()[0];
+        let result = stack![
+            Axis(0),
+            self.0.view().into_shape((1,d)).unwrap(),
+            self.1.view().into_shape((1,d)).unwrap()];
         result
     }
 }
